GCC_PATH = "C:/arm-gcc"

class Target
	attr_reader :name, :cflags, :ldflags
	def initialize(name,cflags, ldflags)
		@name = name
		@cflags = cflags
		@ldflags = ldflags
	end
end

class Platform
	attr_reader :name, :cflags, :ldflags, :cxxflags
	def initialize(name,cflags, ldflags, cxxflags)
		@name = name
		@cflags = cflags
		@ldflags = ldflags
		@cxxflags = cxxflags
	end
end

target = Target.new("Debug",["-g","-O0"],["-g","-O0"])
#Target.new("Debug",["-g","-O0"],["-g","-O0"])
platform = Platform.new("STM32F4",
	["-mthumb", "-mcpu=cortex-m4", "-march=armv7e-m", "-mfpu=fpv4-sp-d16", "-mfloat-abi=hard", "-DUSE_STM32F4_DISCOVERY", "-DSTM32F4XX", "-DUSE_STDPERIPH_DRIVER",  "-ffunction-sections", "-fdata-sections"],
	["-mthumb", "-mcpu=cortex-m4", "-march=armv7e-m", "-mfpu=fpv4-sp-d16", "-mfloat-abi=hard", "-Tstm32_flash.ld", "-static", "-Wl,-cref,-u,Reset_Handler", "-Wl,-Map=test.map", "-Wl,--gc-sections", "-Wl,--defsym=malloc_getpagesize_P=0x1000", "-ffunction-sections"],
	["-fno-exceptions", "-fno-rtti"])
#  "-fdata-sections", GCC_PATH + "/lib/gcc/arm-none-eabi/4.6.2/armv7e-m/fpu/crti.o"])

CC = GCC_PATH + "/bin/arm-none-eabi-gcc"
CXX = GCC_PATH + "/bin/arm-none-eabi-g++"
OBJCOPY = GCC_PATH + "/bin/arm-none-eabi-objcopy"
OBJDUMP = GCC_PATH + "/bin/arm-none-eabi-objdump"

CCFLAGS = ["-Wall", "-std=c99"]
CXXFLAGS = ["-Wall"]
LDFLAGS = [""]

root = Dir.new(".")

def collectFiles(d,p)
	d.entries.inject([]) {|prev,e|
		prev + if(e != "." && e != "..")
			e2 = p=="." ? e : p + "/" + e
			if(File.directory?(e2))
				if(e2==".git")
					[]
				else
					Dir.open(e2) { |sd|
						collectFiles(sd, e2)
					}
				end
			else
				[[e,p]]
			end
		else
			[]
		end
	}
end

files = collectFiles(root,".")

cFiles = files.select{|f| File.extname(f[0]) == ".c" || File.extname(f[0]) == ".S"}
ccFiles = files.select{|f| File.extname(f[0]) == ".cc"}
incDirs = files.select{|f| File.extname(f[0]) == ".h" }.map{|f| f[1]}.uniq

incFlags = incDirs.inject([]) {|p,n| p + ["-I#{n}"]}
ofiles = []

cFiles.each { |f|
	ofile = target.name + "/" + File.basename(f[0],".c") + ".o"
	ofiles = ofiles + [ofile]
	cmd = ([CC, "-c", "-o", ofile, f[1] + "/" + f[0]] + incFlags + CCFLAGS + target.cflags + platform.cflags).join(" ")
	puts cmd
	Kernel.system(cmd)
	if (!$?.success?)
#		puts out
		exit
	end
}

ccFiles.each { |f|
	ofile = target.name + "/" + File.basename(f[0],".cc") + ".o"
	ofiles = ofiles + [ofile]
	cmd = ([CXX, "-c", "-o", ofile, f[1] + "/" + f[0]] + incFlags + CXXFLAGS + target.cflags + platform.cflags + platform.cxxflags).join(" ")
	puts cmd
	Kernel.system(cmd)
	if (!$?.success?)
#		puts out
		exit
	end
}

cmd = ([CC, "-o", target.name + "/test.elf", LDFLAGS + target.ldflags + platform.ldflags] + ofiles).join(" ")
puts cmd
Kernel.system(cmd)

cmd = ([OBJCOPY, "-O", "ihex", target.name + "/test.elf", target.name + "/test.hex"]).join(" ")
puts cmd
Kernel.system(cmd)

cmd = ([OBJCOPY, "-O", "binary", target.name + "/test.elf", target.name + "/test.bin"]).join(" ")
puts cmd
Kernel.system(cmd)

cmd = ([OBJDUMP, "-d", "-S", "-t", target.name + "/test.elf", ">", "Debug/test.disasm"]).join(" ")
puts cmd
Kernel.system(cmd)
