/*
 * bufcomm.h
 *
 *  Created on: 06.05.2012
 *      Author: erlkoenig
 */

#ifndef BUFCOMM_H_
#define BUFCOMM_H_

#include <stdint.h>

class RingBuffer {
	public:
		typedef uint16_t len_t;

		RingBuffer (len_t size, uint8_t* mem);

		inline len_t size () { return m_size; }
		inline len_t fill () { return m_fill; }
		inline len_t available () { return m_size - m_fill; }

		void write (const uint8_t* data, len_t len);
		void read (uint8_t* buffer, len_t len);

		void write_byte (uint8_t byte);
		uint8_t read_byte ();
		void clear ();
	protected:
		len_t m_size, m_fill, m_wptr, m_rptr;
		uint8_t* m_buffer;
};

class BufferedComm {
	public:
		BufferedComm ();

		void on_rx (uint8_t data);
		void on_tx_idle ();
		void on_timer ();
		void reset ();

		inline RingBuffer& in () { return m_in; }
		inline RingBuffer& out () { return m_out; }
	protected:
		// Specialisations

		void set_idle (bool enable);
		bool tx_idle ();
		void write (uint8_t data);
		void set_timer (bool enable);
		void on_rec ();
		void on_sent ();
	protected:
		bool m_timeout;
		uint8_t m_istate, m_ostate;
		RingBuffer::len_t m_peer_size, m_sent_size, m_rec_size, m_send_size;
		RingBuffer m_in, m_out;
		uint8_t m_buf_in [100], m_buf_out[100];
};

#endif /* BUFCOMM_H_ */
