/*
 * t_accel.c
 *
 *  Created on: Dec 27, 2011
 *      Author: erlkoenig
 *
 *
 *  Uses TIM9
 */

#include "stm32f4xx.h"
#include "stm32f4xx_spi.h"
#include "t_accel.h"


 /*
  * Accelerometer
  * 	SPI1
  * 	SCK PA5
  * 	MISO=SDO PA6
  * 	MOSI=SDI PA7
  * 	CS PE3
  * 	CPHA = 1, CPOL = 1
  */

TAccel theAccel;

int8_t TAccelInit () {
	theAccel.fill = 0;
	theAccel.state = 0;
	theAccel.starting = 1;
	theAccel.x_off = 0;
	theAccel.y_off = 0;
	theAccel.z_off = 0;
	/*
	 * SPI
	 */
	// Initialize clocks
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_SPI1, ENABLE);
	RCC_AHB1PeriphClockCmd(RCC_AHB1ENR_GPIOAEN, ENABLE);
	RCC_AHB1PeriphClockCmd(RCC_AHB1ENR_GPIOEEN, ENABLE);

	// Initialize GPIO Pins
	// SCK
	GPIO_InitTypeDef  GPIO_InitStructure;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	GPIO_PinAFConfig (GPIOA, GPIO_PinSource5, GPIO_AF_SPI1);

	// MISO
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	GPIO_PinAFConfig (GPIOA, GPIO_PinSource6, GPIO_AF_SPI1);

	// MOSI
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	GPIO_PinAFConfig (GPIOA, GPIO_PinSource7, GPIO_AF_SPI1);

	// SS
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3;
	GPIO_Init(GPIOE, &GPIO_InitStructure);
	GPIOE->BSRRL = GPIO_Pin_3;

	// Initialize SPI
	SPI_InitTypeDef SpiInit;
	SPI_StructInit (&SpiInit);
	SpiInit.SPI_CPHA = SPI_CPHA_2Edge;
	SpiInit.SPI_CPOL = SPI_CPOL_High;
	SpiInit.SPI_DataSize = SPI_DataSize_16b;
	SpiInit.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
	SpiInit.SPI_FirstBit = SPI_FirstBit_MSB;
	SpiInit.SPI_Mode = SPI_Mode_Master;
	SpiInit.SPI_NSS = SPI_NSS_Soft;
	SpiInit.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_16; // 5,25 MHz
	SPI_CalculateCRC (SPI1, DISABLE);

	NVIC_InitTypeDef NVIC_InitStructure;
	NVIC_InitStructure.NVIC_IRQChannel = SPI1_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	SPI_I2S_ITConfig (SPI1, SPI_I2S_IT_RXNE, DISABLE);
	SPI_I2S_ITConfig (SPI1, SPI_I2S_IT_TXE, DISABLE);

	SPI_Init (SPI1, &SpiInit);
	SPI_Cmd (SPI1, ENABLE);

	if (TAccelRegRead (0x0F) == 0x3B) {
//		GPIOD->BSRRL = GPIO_Pin_15;
		TAccelRegWrite (0x20, 0b01000111);
//		TAccelRegWrite (0x21, 0b00010000);
		TAccelRegWrite (0x21, 0b00000000);
		TAccelRegWrite (0x22, 0);
		TAccelRegWrite (0x38, 0);

		/*
		 * Timer
		 */
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM9, ENABLE);
		TIM_TimeBaseInitTypeDef timBase;

		TIM_TimeBaseStructInit (&timBase);
		timBase.TIM_Prescaler = ((SystemCoreClock) / 28000)-1;						// 10 kHz
		timBase.TIM_ClockDivision = TIM_CKD_DIV1;
		timBase.TIM_CounterMode = TIM_CounterMode_Up;
		timBase.TIM_Period = 280;
		TIM_TimeBaseInit (TIM9, &timBase);

		NVIC_InitStructure.NVIC_IRQChannel = TIM1_BRK_TIM9_IRQn;
		NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
		NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
		NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
		NVIC_Init(&NVIC_InitStructure);

		/*
		 * Synchronize with accelerometer sampling clock
		 */

		for (uint8_t i = 0; i < 2; i++) {
			while ((TAccelRegRead (0x27) & (1 << 3)) == 0);
			TAccelRegRead (TACCEL_REG_X);
			TAccelRegRead (TACCEL_REG_Y);
			TAccelRegRead (TACCEL_REG_Z);
		}

		// Start timer
		TIM_ITConfig(TIM9, TIM_IT_Update, ENABLE);
		TIM9->EGR |= (1 << TIM_EGR_UG);
		TIM_Cmd (TIM9, ENABLE);

		return 1;
	} else {
		return 0;
	}
}

void TAccelDeInit () {
	// DeInitialize GPIO Pins
	// SCK
	GPIO_InitTypeDef  GPIO_InitStructure;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	GPIOA->BSRRL = GPIO_Pin_5;

	// MISO
	GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	// MOSI
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	// SS
	GPIOE->BSRRL = GPIO_Pin_3;

	SPI_Cmd (SPI1, DISABLE);
	SPI_DeInit (SPI1);
	// Stop timer
	TIM_Cmd (TIM9, DISABLE);
	TIM_DeInit (TIM9);
}

void TAccelRegReadAsyncQry (uint8_t address) {
	uint16_t send = 0b1000000000000000 | (((uint16_t) address) << 8);
	SPI_I2S_SendData (SPI1, send);
}

uint8_t TAccelRegRead (uint8_t address) {
	GPIOE->BSRRH = GPIO_Pin_3;
	SPI_I2S_ITConfig (SPI1, SPI_I2S_IT_RXNE, DISABLE);
	SPI_I2S_ITConfig (SPI1, SPI_I2S_IT_TXE, DISABLE);
	TAccelRegReadAsyncQry (address);

	while (SPI_I2S_GetFlagStatus (SPI1, SPI_I2S_FLAG_RXNE) == RESET);

	uint8_t rec = TAccelReadAsyncFetch ();

	while (SPI_I2S_GetFlagStatus (SPI1, SPI_I2S_FLAG_BSY) == SET);

	GPIOE->BSRRL = GPIO_Pin_3;

	return rec;
}

void TAccelRegWrite (uint8_t address, uint8_t data) {
	GPIOE->BSRRH = GPIO_Pin_3;
	uint16_t send = (((uint16_t) address) << 8) | ((uint16_t) data);
	SPI_I2S_SendData (SPI1, send);

	while (SPI_I2S_GetFlagStatus (SPI1, SPI_I2S_FLAG_BSY) == SET);

	GPIOE->BSRRL = GPIO_Pin_3;
}


void TAccelRead (int* x, int* y, int* z) {
	uint8_t ux = TAccelRegRead (0x29), uy = TAccelRegRead (0x2B), uz = TAccelRegRead (0x2D);
	*x = *((int8_t*) &ux);
	*y = *((int8_t*) &uy);
	*z = *((int8_t*) &uz);
}

void SPI1_IRQHandler (void) {
	uint8_t ok = 0;
	if (SPI_I2S_GetFlagStatus (SPI1, SPI_I2S_FLAG_RXNE) == SET) {
		GPIOE->BSRRL = GPIO_Pin_3;
		ok = 1;
		int8_t rec = TAccelReadAsyncFetchI();
		while (SPI_I2S_GetFlagStatus (SPI1, SPI_I2S_FLAG_BSY) == SET);
		__asm__("nop");
		__asm__("nop");
		switch (theAccel.state) {
			case 1:
				GPIOE->BSRRH = GPIO_Pin_3;
				__asm__("nop");
				theAccel.buffer[theAccel.fill * 3] = rec - theAccel.x_off;
				theAccel.state++;
				TAccelRegReadAsyncQry (TACCEL_REG_Y);
				break;
			case 2:
				GPIOE->BSRRH = GPIO_Pin_3;
				__asm__("nop");
				theAccel.buffer[theAccel.fill * 3 + 1] = rec - theAccel.y_off;
				while (SPI_I2S_GetFlagStatus (SPI1, SPI_I2S_FLAG_BSY) == SET);
				theAccel.state++;
				TAccelRegReadAsyncQry (TACCEL_REG_Z);
				break;
			case 3:
				SPI_I2S_ITConfig (SPI1, SPI_I2S_IT_RXNE, DISABLE);
				theAccel.buffer[theAccel.fill * 3 + 2] = rec - theAccel.z_off;
				while (SPI_I2S_GetFlagStatus (SPI1, SPI_I2S_FLAG_BSY) == SET);
				theAccel.fill++;

				/*
				 * Calculate offset (gravity)
				 */

//				GPIOD->BSRRH = GPIO_Pin_15;
				if (theAccel.starting && theAccel.fill == TACCEL_BUF_SIZ) {
					theAccel.fill = 0;
					theAccel.starting = 0;
/*					float x = 0, y = 0, z = 0;
					uint16_t p = 0;
					for (uint16_t i = 0; i < TACCEL_BUF_SIZ; i++) {
						x += theAccel.buffer [p]; p++;
						y += theAccel.buffer [p]; p++;
						z += theAccel.buffer [p]; p++;
					}
					theAccel.x_off = x / ((float) TACCEL_BUF_SIZ);
					theAccel.y_off = y / ((float) TACCEL_BUF_SIZ);
					theAccel.z_off = z / ((float) TACCEL_BUF_SIZ); */
				}
				theAccel.state = 0;
				break;
			default:
				ok = 0;
				break;
		}
	}
	if (!ok) {
		// In case of error interrupt, clear error flags
		int8_t rec = SPI1->DR;
		int8_t status = SPI1->SR;
		status = rec + status;
	}
}

void TIM1_BRK_TIM9_IRQHandler (void) {
	if (TIM9->SR & TIM_SR_UIF) {
		if (theAccel.state == 0) {
			GPIOE->BSRRH = GPIO_Pin_3;
			if (theAccel.fill == TACCEL_BUF_SIZ)
				theAccel.fill = 0;
			theAccel.state = 1;
			TAccelRegReadAsyncQry (TACCEL_REG_X);
			SPI_I2S_ITConfig (SPI1, SPI_I2S_IT_RXNE, ENABLE);
//			GPIOD->BSRRL = GPIO_Pin_15;
		}
		TIM_ClearITPendingBit(TIM9, TIM_IT_Update);
	}
}

int8_t TAccelReadAsyncFetchI() {
	uint16_t r = SPI_I2S_ReceiveData (SPI1) & 0x00FF;
	return *((int8_t*) (&r));
}

uint8_t TAccelReadAsyncFetch() {
	uint16_t r = SPI_I2S_ReceiveData (SPI1) & 0x00FF;
	return *((uint8_t*) (&r));
}
