/*
 * timer.c
 *
 *  Created on: 07.05.2012
 *      Author: erlkoenig
 */

#include "stm32f4xx.h"
#include "timer.h"
#include "servo.h"

#define TIMEOUT_TIM TIM10

extern Timeout timeoutUart;

const uint8_t timeout_count = 1;
Timeout* Timeouts [1] = { &timeoutUart };

void timeouts_init () {
	for (uint8_t i = 0; i < timeout_count; i++) {
		Timeouts[i]->millis = 0;
	}

	NVIC_InitTypeDef NVIC_InitStructure;

	// TimeBase

	RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM10, ENABLE);
	TIM_TimeBaseInitTypeDef timBase;

	TIM_TimeBaseStructInit (&timBase);
	timBase.TIM_Prescaler = ((SystemCoreClock) / 1000000)-1;
	timBase.TIM_ClockDivision = TIM_CKD_DIV1;
	timBase.TIM_CounterMode = TIM_CounterMode_Up;
	timBase.TIM_Period = 1000;
	TIM_TimeBaseInit (TIMEOUT_TIM, &timBase);

	NVIC_InitStructure.NVIC_IRQChannel = TIM1_UP_TIM10_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	// Start timer
	TIM_ITConfig(TIMEOUT_TIM, TIM_IT_Update, ENABLE);
	TIMEOUT_TIM->EGR |= (1 << TIM_EGR_UG);
	TIM_Cmd (TIMEOUT_TIM, ENABLE);
}


void TIM1_UP_TIM10_IRQHandler () {
	if (TIMEOUT_TIM->SR & TIM_SR_UIF) {
		for (uint8_t i = 0; i < timeout_count; i++) {
			Timeout* to = Timeouts[i];
			if (to->millis != 0) {
				to->millis--;
				if (to->millis == 0) {
					(*(to->fun)) ();
				}
			}
		}
		TIM_ClearITPendingBit (TIMEOUT_TIM, TIM_IT_Update);
	}
}
