/*
 * uart.c
 *
 *  Created on: May 6, 2012
 *      Author: erlkoenig
 */

#include "stm32f4xx.h"
#include "uart.h"
#include "timer.h"

BufferedComm usartBufComm;

Timeout timeoutUart;

void uart_init () {
	// PCLK2 = 84 MHz
	// +-- > USART1


	RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);
	RCC_APB2PeriphClockLPModeCmd(RCC_APB2Periph_USART1, ENABLE);
	RCC_AHB1PeriphClockCmd(RCC_AHB1ENR_GPIOBEN, ENABLE);

	GPIO_InitTypeDef GPIO_InitStructure;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOB, &GPIO_InitStructure);
	GPIO_PinAFConfig (GPIOB, GPIO_PinSource6, GPIO_AF_USART1);

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7;
	GPIO_Init(GPIOB, &GPIO_InitStructure);
	GPIO_PinAFConfig (GPIOB, GPIO_PinSource7, GPIO_AF_USART1);


	USART_InitTypeDef uInit;
	USART_StructInit (&uInit);
	uInit.USART_BaudRate = 57600;
	uInit.USART_WordLength = USART_WordLength_8b;
	uInit.USART_StopBits = USART_StopBits_1;
	uInit.USART_Parity = USART_Parity_No;
	uInit.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
	uInit.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
	USART_Init (USART1, &uInit);
	USART1->BRR = 0x5B2;

	USART_ClockInitTypeDef ucInit;
	USART_ClockStructInit (&ucInit);
	ucInit.USART_Clock = USART_Clock_Disable;
	USART_ClockInit (USART1, &ucInit);

	USART_Cmd (USART1, ENABLE);

	NVIC_InitTypeDef NVIC_InitStructure;
	NVIC_InitStructure.NVIC_IRQChannel = USART1_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	USART_ITConfig (USART1, USART_IT_TXE, ENABLE);
	USART_ITConfig (USART1, USART_IT_RXNE, ENABLE);

	timeoutUart.fun = &uart_timeout_fun;
	timeoutUart.millis = 1000;
}


extern "C" {
void USART1_IRQHandler (void) {
	if (USART_GetFlagStatus (USART1, USART_FLAG_TXE) == SET) {
		usartBufComm.on_tx_idle ();
	}
	if (USART_GetFlagStatus (USART1, USART_FLAG_RXNE) == SET) {
		usartBufComm.on_rx (USART_ReceiveData (USART1));
	}
}

void uart_timeout_fun () {
	usartBufComm.on_timer ();
}
};

// ---------
// BufferedComm Specialisations

void BufferedComm::set_idle (bool enable) {
	USART_ITConfig (USART1, USART_IT_TXE, enable ? ENABLE : DISABLE);
}

bool BufferedComm::tx_idle () {
	return USART_GetFlagStatus (USART1, USART_FLAG_TXE) == SET;
}

void BufferedComm::write (uint8_t data) {
	USART_SendData (USART1, data);
}

void BufferedComm::set_timer (bool enable) {
	timeoutUart.millis = enable ? 100 : 0;
}

void BufferedComm::on_rec () {

}

void BufferedComm::on_sent () {

}
